@@ -8,6 +8,7 @@
 #include "FlatpakBackend.h"
 #include "FlatpakFetchDataJob.h"
 #include "FlatpakJobTransaction.h"
+#include "FlatpakRefreshAppstreamMetadataJob.h"
 #include "FlatpakSourcesBackend.h"
 
 #include <ReviewsBackend/Rating.h>
@@ -134,6 +135,11 @@ public:
         });
     }
 
+    FlatpakRemote *remote() const
+    {
+        return m_remote;
+    }
+
     AppStream::Pool *m_pool = nullptr;
     QHash<FlatpakResource::Id, FlatpakResource *> m_resources;
 
@@ -169,9 +175,6 @@ static FlatpakResource::Id idForRefString(const QStringView &ref)
 
 static FlatpakResource::Id idForInstalledRef(FlatpakInstalledRef *ref, const QString &postfix)
 {
-    const FlatpakResource::ResourceType appType = (flatpak_ref_get_kind(FLATPAK_REF(ref)) == FLATPAK_REF_KIND_APP //
-                                                       ? FlatpakResource::DesktopApp
-                                                       : FlatpakResource::Runtime);
     const QString appId = QLatin1String(flatpak_ref_get_name(FLATPAK_REF(ref))) + postfix;
     const QString arch = QString::fromUtf8(flatpak_ref_get_arch(FLATPAK_REF(ref)));
     const QString branch = QString::fromUtf8(flatpak_ref_get_branch(FLATPAK_REF(ref)));
@@ -581,6 +584,96 @@ void FlatpakBackend::addAppFromFlatpakBundle(const QUrl &url, ResultsStream *str
     stream->resourcesFound({resource});
 }
 
+AppStream::Component fetchComponentFromRemote(const QSettings &settings, GCancellable *cancellable)
+{
+    const QString name = settings.value(QStringLiteral("Flatpak Ref/Name")).toString();
+    const QString remoteName = settings.value(QStringLiteral("Flatpak Ref/SuggestRemoteName")).toString();
+
+    AppStream::Component asComponent;
+    asComponent.addUrl(AppStream::Component::UrlKindHomepage, settings.value(QStringLiteral("Flatpak Ref/Homepage")).toString());
+    asComponent.setDescription(settings.value(QStringLiteral("Flatpak Ref/Description")).toString());
+    asComponent.setName(settings.value(QStringLiteral("Flatpak Ref/Title")).toString());
+    asComponent.setSummary(settings.value(QStringLiteral("Flatpak Ref/Comment")).toString());
+    asComponent.setId(name);
+
+    AppStream::Bundle b;
+    b.setKind(AppStream::Bundle::KindFlatpak);
+    b.setId(asComponent.name());
+    asComponent.addBundle(b);
+
+    // We are going to create a temporary installation and add the remote to it.
+    // There we will fetch the appstream metadata and then delete that temporary installation.
+
+    g_autoptr(GError) localError = nullptr;
+    const QString path = QStandardPaths::writableLocation(QStandardPaths::TempLocation) + QLatin1String("/discover-flatpak-temporary") + remoteName;
+    qDebug() << "Creating temporary installation" << path;
+    g_autoptr(GFile) file = g_file_new_for_path(QFile::encodeName(path).constData());
+    g_autoptr(FlatpakInstallation) tempInstallation = flatpak_installation_new_for_path(file, true, cancellable, &localError);
+    if (!tempInstallation) {
+        return asComponent;
+    }
+    auto x = qScopeGuard([path] {
+        QDir(path).removeRecursively();
+    });
+
+    g_autoptr(FlatpakRemote) tempRemote = flatpak_remote_new(remoteName.toUtf8());
+    FlatpakSourcesBackend::populateRemote(tempRemote,
+                                          remoteName,
+                                          settings.value(QStringLiteral("Flatpak Ref/Url")).toString(),
+                                          settings.value(QStringLiteral("Flatpak Ref/GPGKey")).toString().toUtf8());
+    if (!flatpak_installation_modify_remote(tempInstallation, tempRemote, cancellable, &localError)) {
+        qDebug() << "error adding temporary remote" << localError->message;
+        return {asComponent};
+    }
+
+    auto cb = [](const char *status, guint progress, gboolean /*estimating*/, gpointer /*user_data*/) {
+        qDebug() << "Progress..." << status << progress;
+    };
+
+    gboolean changed;
+    if (!flatpak_installation_update_appstream_full_sync(tempInstallation, remoteName.toUtf8(), nullptr, cb, nullptr, &changed, cancellable, &localError)) {
+        qDebug() << "error fetching appstream" << localError->message;
+        return {asComponent};
+    }
+    Q_ASSERT(changed);
+    const QString appstreamLocation = path + "/appstream/" + remoteName + '/' + flatpak_get_default_arch() + "/active";
+
+    AppStream::Pool pool;
+#ifdef APPSTREAM_NEW_POOL_API
+    pool.setLoadStdDataLocations(false);
+    pool.addExtraDataLocation(appstreamLocation, AppStream::Metadata::FormatStyleCollection);
+#else
+    pool.clearMetadataLocations();
+    pool.addMetadataLocation(appstreamLocation);
+    pool.setFlags(AppStream::Pool::FlagReadCollection);
+    pool.setCacheFlags(AppStream::Pool::CacheFlagUseUser);
+
+    const QString subdir = flatpak_installation_get_id(tempInstallation) + QLatin1Char('/') + remoteName;
+    pool.setCacheLocation(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + "/flatpak-appstream-temp/" + subdir);
+    QDir().mkpath(pool.cacheLocation());
+#endif
+
+    if (!pool.load()) {
+        qDebug() << "error loading pool" << pool.lastError();
+        return {asComponent};
+    }
+
+    const QString branch = settings.value(QStringLiteral("Flatpak Ref/Branch")).toString();
+
+    auto comps = pool.componentsById(name);
+    if (!branch.isEmpty()) {
+        const QString suffix = QLatin1Char('/') + branch;
+        comps = kFilter<QList<AppStream::Component>>(comps, [&suffix](const AppStream::Component &component) {
+            return component.bundle(AppStream::Bundle::KindFlatpak).id().endsWith(suffix);
+        });
+    }
+    if (comps.isEmpty()) {
+        qDebug() << "could not find" << name << "in" << remoteName;
+        return asComponent;
+    }
+    return comps.constFirst();
+}
+
 void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream)
 {
     Q_ASSERT(url.isLocalFile());
@@ -620,44 +713,7 @@ void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream
         }
     }
 
-    g_autoptr(FlatpakRemoteRef) remoteRef = nullptr;
-    {
-        QFile f(url.toLocalFile());
-        if (!f.open(QFile::ReadOnly | QFile::Text)) {
-            stream->finish();
-            return;
-        }
-
-        QByteArray contents = f.readAll();
-
-        g_autoptr(GBytes) bytes = g_bytes_new(contents.data(), contents.size());
-
-        remoteRef = flatpak_installation_install_ref_file(preferredInstallation(), bytes, m_cancellable, &error);
-        if (!remoteRef) {
-            qWarning() << "Failed to create install ref file:" << error->message;
-            AbstractResourcesBackend::Filters filter;
-            filter.resourceUrl = QUrl(QLatin1String("appstream://") + name);
-            auto streamKnown = search(filter);
-            connect(streamKnown, &ResultsStream::resourcesFound, stream, &ResultsStream::resourcesFound);
-            connect(streamKnown, &ResultsStream::destroyed, stream, &ResultsStream::finish);
-            return;
-        }
-    }
-
-    auto ref = FLATPAK_REF(remoteRef);
-
-    AppStream::Component asComponent;
-    asComponent.addUrl(AppStream::Component::UrlKindHomepage, settings.value(QStringLiteral("Flatpak Ref/Homepage")).toString());
-    asComponent.setDescription(settings.value(QStringLiteral("Flatpak Ref/Description")).toString());
-    asComponent.setName(settings.value(QStringLiteral("Flatpak Ref/Title")).toString());
-    asComponent.setSummary(settings.value(QStringLiteral("Flatpak Ref/Comment")).toString());
-    asComponent.setId(name);
-
-    AppStream::Bundle b;
-    b.setKind(AppStream::Bundle::KindFlatpak);
-    b.setId(flatpak_ref_format_ref(ref));
-    asComponent.addBundle(b);
-
+    AppStream::Component asComponent = fetchComponentFromRemote(settings, m_cancellable);
     const QString iconUrl = settings.value(QStringLiteral("Flatpak Ref/Icon")).toString();
     if (!iconUrl.isEmpty()) {
         AppStream::Icon icon;
@@ -668,8 +724,13 @@ void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream
 
     auto resource = new FlatpakResource(asComponent, preferredInstallation(), this);
     resource->setFlatpakFileType(FlatpakResource::FileFlatpakRef);
+    resource->setResourceFile(url);
+    resource->setResourceLocation(refurl);
     resource->setOrigin(remoteName);
-    resource->updateFromRef(ref);
+    resource->setFlatpakName(name);
+    resource->setArch(flatpak_get_default_arch());
+    resource->setBranch(settings.value(QStringLiteral("Flatpak Ref/Branch")).toString());
+    resource->setType(settings.value(QStringLiteral("Flatpak Ref/IsRuntime")).toBool() ? FlatpakResource::Runtime : FlatpakResource::DesktopApp);
 
     QUrl runtimeUrl = QUrl(settings.value(QStringLiteral("Flatpak Ref/RuntimeRepo")).toString());
     auto refSource = QSharedPointer<FlatpakSource>::create(this, preferredInstallation());
@@ -796,29 +857,53 @@ bool FlatpakBackend::loadAppsFromAppstreamData(FlatpakInstallation *flatpakInsta
         return false;
     }
 
-    m_refreshAppstreamMetadataJobs += remotes->len;
-
     for (uint i = 0; i < remotes->len; i++) {
         FlatpakRemote *remote = FLATPAK_REMOTE(g_ptr_array_index(remotes, i));
-        g_autoptr(GFile) fileTimestamp = flatpak_remote_get_appstream_timestamp(remote, flatpak_get_default_arch());
-
-        g_autofree char *path_str = g_file_get_path(fileTimestamp);
-        QFileInfo fileInfo(QFile::encodeName(path_str));
-        // Refresh appstream metadata in case they have never been refreshed or the cache is older than 6 hours
-        if (!fileInfo.exists() || fileInfo.lastModified().toUTC().secsTo(QDateTime::currentDateTimeUtc()) > 21600) {
-            refreshAppstreamMetadata(flatpakInstallation, remote);
-        } else {
-            integrateRemote(flatpakInstallation, remote);
-        }
+        loadRemote(flatpakInstallation, remote);
     }
     return true;
 }
 
+void FlatpakBackend::loadRemote(FlatpakInstallation *installation, FlatpakRemote *remote)
+{
+    g_autoptr(GFile) fileTimestamp = flatpak_remote_get_appstream_timestamp(remote, flatpak_get_default_arch());
+
+    g_autofree char *path_str = g_file_get_path(fileTimestamp);
+    QFileInfo fileInfo(QFile::encodeName(path_str));
+    // Refresh appstream metadata in case they have never been refreshed or the cache is older than 6 hours
+    if (!fileInfo.exists() || fileInfo.lastModified().toUTC().secsTo(QDateTime::currentDateTimeUtc()) > 21600) {
+        checkForUpdates(installation, remote);
+    } else {
+        m_refreshAppstreamMetadataJobs++;
+        integrateRemote(installation, remote);
+    }
+}
+
+void FlatpakBackend::unloadRemote(FlatpakInstallation *installation, FlatpakRemote *remote)
+{
+    acquireFetching(true);
+    for (auto it = m_flatpakSources.begin(); it != m_flatpakSources.end();) {
+        if ((*it)->url() == flatpak_remote_get_url(remote) && (*it)->installation() == installation) {
+            qDebug() << "unloading remote" << (*it) << remote;
+            it = m_flatpakSources.erase(it);
+        } else {
+            ++it;
+        }
+    }
+    acquireFetching(false);
+}
+
 void FlatpakBackend::metadataRefreshed()
 {
     m_refreshAppstreamMetadataJobs--;
     if (m_refreshAppstreamMetadataJobs == 0) {
-        checkForUpdates();
+        for (auto installation : qAsConst(m_installations)) {
+            // Load local updates, comparing current and latest commit
+            loadLocalUpdates(installation);
+
+            if (g_cancellable_is_cancelled(m_cancellable))
+                break;
+        }
     }
 }
 
@@ -827,14 +912,12 @@ QSharedPointer<FlatpakSource> FlatpakBackend::integrateRemote(FlatpakInstallatio
     Q_ASSERT(m_refreshAppstreamMetadataJobs != 0);
     for (auto source : qAsConst(m_flatpakSources)) {
         if (source->url() == flatpak_remote_get_url(remote) && source->installation() == flatpakInstallation) {
-            qDebug() << "do not add a source twice" << source << remote;
             metadataRefreshed();
             return source;
         }
     }
     for (auto source : qAsConst(m_flatpakLoadingSources)) {
         if (source->url() == flatpak_remote_get_url(remote) && source->installation() == flatpakInstallation) {
-            qDebug() << "do not add a source twice" << source << remote;
             metadataRefreshed();
             return source;
         }
@@ -870,6 +953,11 @@ QSharedPointer<FlatpakSource> FlatpakBackend::integrateRemote(FlatpakInstallatio
         fw->deleteLater();
     });
     acquireFetching(true);
+
+#ifdef APPSTREAM_NEW_POOL_API
+    pool->setLoadStdDataLocations(false);
+    pool->addExtraDataLocation(appstreamDirPath, AppStream::Metadata::FormatStyleCollection);
+#else
     pool->clearMetadataLocations();
     pool->addMetadataLocation(appstreamDirPath);
     pool->setFlags(AppStream::Pool::FlagReadCollection);
@@ -878,6 +966,8 @@ QSharedPointer<FlatpakSource> FlatpakBackend::integrateRemote(FlatpakInstallatio
     const QString subdir = flatpak_installation_get_id(flatpakInstallation) + QLatin1Char('/') + sourceName;
     pool->setCacheLocation(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + "/flatpak-appstream/" + subdir);
     QDir().mkpath(pool->cacheLocation());
+#endif
+
     fw->setFuture(QtConcurrent::run(&m_threadPool, pool, &AppStream::Pool::load));
     m_flatpakLoadingSources << source;
     return source;
@@ -929,83 +1019,6 @@ bool FlatpakBackend::parseMetadataFromAppBundle(FlatpakResource *resource)
     return true;
 }
 
-class FlatpakRefreshAppstreamMetadataJob : public QThread
-{
-    Q_OBJECT
-public:
-    FlatpakRefreshAppstreamMetadataJob(FlatpakInstallation *installation, FlatpakRemote *remote)
-        : QThread()
-        , m_cancellable(g_cancellable_new())
-        , m_installation(installation)
-        , m_remote(remote)
-    {
-        g_object_ref(m_remote);
-        connect(this, &FlatpakRefreshAppstreamMetadataJob::finished, this, &QObject::deleteLater);
-    }
-
-    ~FlatpakRefreshAppstreamMetadataJob()
-    {
-        g_object_unref(m_remote);
-        g_object_unref(m_cancellable);
-    }
-
-    void cancel()
-    {
-        g_cancellable_cancel(m_cancellable);
-    }
-
-    void run() override
-    {
-        g_autoptr(GError) localError = nullptr;
-
-#if FLATPAK_CHECK_VERSION(0, 9, 4)
-        // With Flatpak 0.9.4 we can use flatpak_installation_update_appstream_full_sync() providing progress reporting which we don't use at this moment, but
-        // still better to use newer function in case the previous one gets deprecated
-        if (!flatpak_installation_update_appstream_full_sync(m_installation,
-                                                             flatpak_remote_get_name(m_remote),
-                                                             nullptr,
-                                                             nullptr,
-                                                             nullptr,
-                                                             nullptr,
-                                                             m_cancellable,
-                                                             &localError)) {
-#else
-        if (!flatpak_installation_update_appstream_sync(m_installation, flatpak_remote_get_name(m_remote), nullptr, nullptr, m_cancellable, &localError)) {
-#endif
-            const QString error = localError ? QString::fromUtf8(localError->message) : QStringLiteral("<no error>");
-            qWarning() << "Failed to refresh appstream metadata for " << flatpak_remote_get_name(m_remote) << ": " << error;
-            Q_EMIT jobRefreshAppstreamMetadataFailed(error);
-        } else {
-            Q_EMIT jobRefreshAppstreamMetadataFinished(m_installation, m_remote);
-        }
-    }
-
-Q_SIGNALS:
-    void jobRefreshAppstreamMetadataFailed(const QString &errorMessage);
-    void jobRefreshAppstreamMetadataFinished(FlatpakInstallation *installation, FlatpakRemote *remote);
-
-private:
-    GCancellable *m_cancellable;
-    FlatpakInstallation *m_installation;
-    FlatpakRemote *m_remote;
-};
-
-void FlatpakBackend::refreshAppstreamMetadata(FlatpakInstallation *installation, FlatpakRemote *remote)
-{
-    FlatpakRefreshAppstreamMetadataJob *job = new FlatpakRefreshAppstreamMetadataJob(installation, remote);
-    connect(job, &FlatpakRefreshAppstreamMetadataJob::jobRefreshAppstreamMetadataFailed, this, &FlatpakBackend::metadataRefreshed);
-    connect(job, &FlatpakRefreshAppstreamMetadataJob::jobRefreshAppstreamMetadataFailed, this, [this](const QString &errorMessage) {
-        Q_EMIT passiveMessage(errorMessage);
-    });
-    connect(job, &FlatpakRefreshAppstreamMetadataJob::jobRefreshAppstreamMetadataFinished, this, &FlatpakBackend::integrateRemote);
-    connect(job, &FlatpakRefreshAppstreamMetadataJob::finished, this, [this] {
-        acquireFetching(false);
-    });
-
-    acquireFetching(true);
-    job->start();
-}
-
 bool FlatpakBackend::setupFlatpakInstallations(GError **error)
 {
     if (qEnvironmentVariableIsSet("FLATPAK_TEST_MODE")) {
@@ -1261,34 +1274,67 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
     else if (filter.state == AbstractResource::Upgradeable) {
         auto stream = new ResultsStream(QStringLiteral("FlatpakStream-upgradeable"));
         auto f = [this, stream] {
-            QVector<AbstractResource *> resources;
-            for (auto installation : m_installations) {
-                g_autoptr(GError) localError = nullptr;
-                g_autoptr(GPtrArray) refs = flatpak_installation_list_installed_refs_for_update(installation, m_cancellable, &localError);
-                if (!refs) {
-                    qWarning() << "Failed to get list of installed refs for listing updates:" << localError->message;
-                    continue;
+            auto fw = new QFutureWatcher<QMap<FlatpakInstallation *, QVector<FlatpakInstalledRef *>>>(this);
+            connect(fw, &QFutureWatcher<QByteArray>::finished, this, [this, fw, stream]() {
+                if (g_cancellable_is_cancelled(m_cancellable)) {
+                    stream->finish();
+                    fw->deleteLater();
+                    return;
                 }
 
-                resources.reserve(resources.size() + refs->len);
-                for (uint i = 0; i < refs->len; i++) {
-                    FlatpakInstalledRef *ref = FLATPAK_INSTALLED_REF(g_ptr_array_index(refs, i));
-                    auto resource = getAppForInstalledRef(installation, ref);
+                const auto refs = fw->result();
+                QVector<AbstractResource *> resources;
+                for (auto it = refs.constBegin(), itEnd = refs.constEnd(); it != itEnd; ++it) {
+                    resources.reserve(resources.size() + it->size());
+                    for (auto ref : qAsConst(it.value())) {
+                        auto resource = getAppForInstalledRef(it.key(), ref);
 #if FLATPAK_CHECK_VERSION(1, 1, 2)
-                    resource->setAvailableVersion(QString::fromUtf8(flatpak_installed_ref_get_appdata_version(ref)));
+                        resource->setAvailableVersion(QString::fromUtf8(flatpak_installed_ref_get_appdata_version(ref)));
 #endif
-                    resource->setState(AbstractResource::Upgradeable);
-                    updateAppSize(resource);
-                    if (resource->resourceType() == FlatpakResource::Runtime) {
-                        resources.prepend(resource);
-                    } else {
-                        resources.append(resource);
+                        g_object_unref(ref);
+                        resource->setState(AbstractResource::Upgradeable);
+                        updateAppSize(resource);
+                        if (resource->resourceType() == FlatpakResource::Runtime) {
+                            resources.prepend(resource);
+                        } else {
+                            resources.append(resource);
+                        }
                     }
                 }
-            }
-            if (!resources.isEmpty())
-                Q_EMIT stream->resourcesFound(resources);
-            stream->finish();
+
+                if (!resources.isEmpty())
+                    Q_EMIT stream->resourcesFound(resources);
+                stream->finish();
+                fw->deleteLater();
+            });
+
+            QVector<FlatpakInstallation *> installations = m_installations;
+            auto cancellable = m_cancellable;
+            fw->setFuture(QtConcurrent::run(&m_threadPool, [installations, cancellable] {
+                QMap<FlatpakInstallation *, QVector<FlatpakInstalledRef *>> ret;
+
+                for (auto installation : installations) {
+                    g_autoptr(GError) localError = nullptr;
+                    g_autoptr(GPtrArray) refs = flatpak_installation_list_installed_refs_for_update(installation, cancellable, &localError);
+                    if (!refs) {
+                        qWarning() << "Failed to get list of installed refs for listing updates:" << localError->message;
+                        continue;
+                    }
+                    if (g_cancellable_is_cancelled(cancellable)) {
+                        qWarning() << "Job cancelled";
+                        return ret;
+                    }
+
+                    auto &current = ret[installation];
+                    current.reserve(refs->len);
+                    for (uint i = 0; i < refs->len; i++) {
+                        FlatpakInstalledRef *ref = FLATPAK_INSTALLED_REF(g_ptr_array_index(refs, i));
+                        g_object_ref(ref);
+                        current.append(ref);
+                    }
+                }
+                return ret;
+            }));
         };
 
         if (isFetching()) {
@@ -1478,12 +1524,11 @@ Transaction *FlatpakBackend::installApplication(AbstractResource *app, const Add
         FlatpakRemote *remote = m_sources->installSource(resource);
         if (remote) {
             resource->setState(AbstractResource::Installed);
-            m_refreshAppstreamMetadataJobs++;
             // Make sure we update appstream metadata first
             // FIXME we have to let flatpak to return the remote as the one created by FlatpakSourcesBackend will not have appstream directory
             g_autoptr(FlatpakRemote) repo =
                 flatpak_installation_get_remote_by_name(resource->installation(), flatpak_remote_get_name(remote), m_cancellable, nullptr);
-            refreshAppstreamMetadata(resource->installation(), repo);
+            loadRemote(resource->installation(), repo);
         }
         return nullptr;
     }
@@ -1526,18 +1571,47 @@ Transaction *FlatpakBackend::removeApplication(AbstractResource *app)
 
 void FlatpakBackend::checkForUpdates()
 {
-    for (auto installation : qAsConst(m_installations)) {
-        // Load local updates, comparing current and latest commit
-        loadLocalUpdates(installation);
-
-        if (g_cancellable_is_cancelled(m_cancellable))
-            break;
+    for (auto source : qAsConst(m_flatpakSources)) {
+        checkForUpdates(source->installation(), source->remote());
     }
 }
 
+void FlatpakBackend::checkForUpdates(FlatpakInstallation *installation, FlatpakRemote *remote)
+{
+    m_refreshAppstreamMetadataJobs++;
+    if (flatpak_remote_get_disabled(remote)) {
+        integrateRemote(installation, remote);
+        return;
+    }
+
+    FlatpakRefreshAppstreamMetadataJob *job = new FlatpakRefreshAppstreamMetadataJob(installation, remote);
+    connect(job, &FlatpakRefreshAppstreamMetadataJob::jobRefreshAppstreamMetadataFailed, this, &FlatpakBackend::metadataRefreshed);
+    connect(job, &FlatpakRefreshAppstreamMetadataJob::jobRefreshAppstreamMetadataFailed, this, &FlatpakBackend::passiveMessage);
+    connect(job, &FlatpakRefreshAppstreamMetadataJob::jobRefreshAppstreamMetadataFinished, this, &FlatpakBackend::integrateRemote);
+    connect(job, &FlatpakRefreshAppstreamMetadataJob::finished, this, [this] {
+        acquireFetching(false);
+    });
+
+    acquireFetching(true);
+    job->start();
+}
+
 QString FlatpakBackend::displayName() const
 {
     return QStringLiteral("Flatpak");
 }
 
+HelpfulError *FlatpakBackend::explainDysfunction() const
+{
+    if (m_flatpakSources.isEmpty()) {
+        return new HelpfulError(QStringLiteral("emblem-error"), i18n("There are no Flatpak sources."), m_sources->actions());
+    }
+    for (const auto &source : m_flatpakSources) {
+        if (!source->m_pool->lastError().isEmpty()) {
+            return new HelpfulError(QStringLiteral("emblem-error"), i18n("Failed to load \"%1\" source", source->name()));
+        }
+    }
+    return AbstractResourcesBackend::explainDysfunction();
+}
+
 #include "FlatpakBackend.moc"
