@@ -73,6 +73,10 @@ public:
 
     bool updateAppSize(FlatpakResource *resource);
     FlatpakInstalledRef *getInstalledRefForApp(const FlatpakResource *resource) const;
+    void loadRemote(FlatpakInstallation *installation, FlatpakRemote *remote);
+    void unloadRemote(FlatpakInstallation *installation, FlatpakRemote *remote);
+
+    HelpfulError *explainDysfunction() const override;
 
 private Q_SLOTS:
     void onFetchMetadataFinished(FlatpakResource *resource, const QByteArray &metadata);
@@ -100,7 +104,6 @@ private:
     bool loadAppsFromAppstreamData(FlatpakInstallation *flatpakInstallation);
     void loadLocalUpdates(FlatpakInstallation *flatpakInstallation);
     bool parseMetadataFromAppBundle(FlatpakResource *resource);
-    void refreshAppstreamMetadata(FlatpakInstallation *installation, FlatpakRemote *remote);
     bool setupFlatpakInstallations(GError **error);
     void updateAppInstalledMetadata(FlatpakInstalledRef *installedRef, FlatpakResource *resource);
     bool updateAppMetadata(FlatpakResource *resource);
@@ -112,6 +115,7 @@ private:
 
     QVector<AbstractResource *> resourcesByAppstreamName(const QString &name) const;
     void acquireFetching(bool f);
+    void checkForUpdates(FlatpakInstallation *flatpakInstallation, FlatpakRemote *remote);
 
     StandardBackendUpdater *m_updater;
     FlatpakSourcesBackend *m_sources = nullptr;
