@@ -31,6 +31,7 @@
 #include <QProcess>
 #include <QStringList>
 #include <QTimer>
+#include <QUrlQuery>
 
 static QString iconCachePath(const AppStream::Icon &icon)
 {
@@ -38,6 +39,8 @@ static QString iconCachePath(const AppStream::Icon &icon)
     return QStringLiteral("%1/icons/%2").arg(QStandardPaths::writableLocation(QStandardPaths::CacheLocation), icon.url().fileName());
 }
 
+const QStringList FlatpakResource::m_objects{QStringLiteral("qrc:/qml/FlatpakAttention.qml")};
+
 FlatpakResource::FlatpakResource(const AppStream::Component &component, FlatpakInstallation *installation, FlatpakBackend *parent)
     : AbstractResource(parent)
     , m_appdata(component)
@@ -228,7 +231,7 @@ QString FlatpakResource::installedVersion() const
     }
 
     if (auto ref = qobject_cast<FlatpakBackend *>(backend())->getInstalledRefForApp(this)) {
-        return i18n("%1 (%2)", flatpak_installed_ref_get_appdata_version(ref), version);
+        return i18nc("version (branch)", "%1 (%2)", flatpak_installed_ref_get_appdata_version(ref), version);
     }
     return version;
 }
@@ -293,7 +296,26 @@ QJsonArray FlatpakResource::licenses()
 
 QString FlatpakResource::longDescription()
 {
-    return m_appdata.description();
+    QString description = m_appdata.description();
+
+    return description;
+}
+
+QString FlatpakResource::attentionText() const
+{
+    if (m_flatpakFileType == FlatpakResource::FileFlatpakRef) {
+        QUrl loc = m_resourceLocation;
+        loc.setPath({});
+        loc.setQuery(QUrlQuery());
+        return xi18nc("@info",
+                      "<para>This application comes from \"%1\" (hosted at <link>%2</link>). Other software in this repository will also be made be available "
+                      "in Discover "
+                      "when the application is "
+                      "installed.</para>",
+                      m_origin,
+                      loc.toDisplayString());
+    }
+    return {};
 }
 
 QString FlatpakResource::name() const
