@@ -24,9 +24,16 @@
 class FlatpakSourceItem : public QStandardItem
 {
 public:
-    FlatpakSourceItem(const QString &text)
+    FlatpakSourceItem(const QString &text, FlatpakRemote *remote, FlatpakBackend *backend)
         : QStandardItem(text)
+        , m_remote(remote)
+        , m_backend(backend)
     {
+        g_object_ref(remote);
+    }
+    ~FlatpakSourceItem()
+    {
+        g_object_unref(m_remote);
     }
 
     void setFlatpakInstallation(FlatpakInstallation *installation)
@@ -39,15 +46,41 @@ public:
         return m_installation;
     }
 
+    void setData(const QVariant &value, int role) override
+    {
+        // We check isCheckable() so the initial setting of the item doesn't trigger a change
+        if (role == Qt::CheckStateRole && isCheckable()) {
+            const bool disabled = flatpak_remote_get_disabled(m_remote);
+            const bool requestedDisabled = Qt::Unchecked == value;
+            if (disabled != requestedDisabled) {
+                flatpak_remote_set_disabled(m_remote, requestedDisabled);
+                g_autoptr(GError) error = nullptr;
+                if (!flatpak_installation_modify_remote(m_installation, m_remote, nullptr, &error)) {
+                    qWarning() << "set disabled failed" << error->message;
+                    return;
+                }
+
+                if (requestedDisabled) {
+                    m_backend->unloadRemote(m_installation, m_remote);
+                } else {
+                    m_backend->loadRemote(m_installation, m_remote);
+                }
+            }
+        }
+        QStandardItem::setData(value, role);
+    }
+
 private:
-    FlatpakInstallation *m_installation;
+    FlatpakInstallation *m_installation = nullptr;
+    FlatpakRemote *const m_remote;
+    FlatpakBackend *const m_backend;
 };
 
 FlatpakSourcesBackend::FlatpakSourcesBackend(const QVector<FlatpakInstallation *> &installations, AbstractResourcesBackend *parent)
     : AbstractSourcesBackend(parent)
     , m_preferredInstallation(installations.constFirst())
     , m_sources(new QStandardItemModel(this))
-    , m_flathubAction(new DiscoverAction(i18n("Add Flathub"), this))
+    , m_flathubAction(new DiscoverAction("flatpak-discover", i18n("Add Flathub"), this))
     , m_noSourcesItem(new QStandardItem(QStringLiteral("-")))
 {
     m_flathubAction->setObjectName(QStringLiteral("flathub"));
@@ -274,22 +307,12 @@ bool FlatpakSourcesBackend::listRepositories(FlatpakInstallation *installation)
     return true;
 }
 
-FlatpakRemote *FlatpakSourcesBackend::installSource(FlatpakResource *resource)
+void FlatpakSourcesBackend::populateRemote(FlatpakRemote *remote, const QString &name, const QString &url, const QString &gpgKey)
 {
-    g_autoptr(GCancellable) cancellable = g_cancellable_new();
-
-    auto remote = flatpak_installation_get_remote_by_name(m_preferredInstallation, resource->flatpakName().toUtf8().constData(), cancellable, nullptr);
-    if (remote) {
-        qWarning() << "Source " << resource->flatpakName() << " already exists in" << flatpak_installation_get_path(m_preferredInstallation);
-        return nullptr;
-    }
-
-    remote = flatpak_remote_new(resource->flatpakName().toUtf8().constData());
-    flatpak_remote_set_url(remote, resource->getMetadata(QStringLiteral("repo-url")).toString().toUtf8().constData());
+    flatpak_remote_set_url(remote, url.toUtf8().constData());
     flatpak_remote_set_noenumerate(remote, false);
-    flatpak_remote_set_title(remote, resource->comment().toUtf8().constData());
+    flatpak_remote_set_title(remote, name.toUtf8().constData());
 
-    const QString gpgKey = resource->getMetadata(QStringLiteral("gpg-key")).toString();
     if (!gpgKey.isEmpty()) {
         gsize dataLen = 0;
         g_autofree guchar *data = nullptr;
@@ -301,13 +324,30 @@ FlatpakRemote *FlatpakSourcesBackend::installSource(FlatpakResource *resource)
     } else {
         flatpak_remote_set_gpg_verify(remote, false);
     }
+}
 
+FlatpakRemote *FlatpakSourcesBackend::installSource(FlatpakResource *resource)
+{
+    g_autoptr(GCancellable) cancellable = g_cancellable_new();
+
+    auto remote = flatpak_installation_get_remote_by_name(m_preferredInstallation, resource->flatpakName().toUtf8().constData(), cancellable, nullptr);
+    if (remote) {
+        qWarning() << "Source " << resource->flatpakName() << " already exists in" << flatpak_installation_get_path(m_preferredInstallation);
+        return nullptr;
+    }
+
+    remote = flatpak_remote_new(resource->flatpakName().toUtf8().constData());
+    populateRemote(remote,
+                   resource->comment(),
+                   resource->getMetadata(QStringLiteral("repo-url")).toString(),
+                   resource->getMetadata(QStringLiteral("gpg-key")).toString());
     if (!resource->branch().isEmpty()) {
         flatpak_remote_set_default_branch(remote, resource->branch().toUtf8().constData());
     }
 
-    if (!flatpak_installation_modify_remote(m_preferredInstallation, remote, cancellable, nullptr)) {
-        qWarning() << "Failed to add source " << resource->flatpakName();
+    g_autoptr(GError) error = nullptr;
+    if (!flatpak_installation_modify_remote(m_preferredInstallation, remote, cancellable, &error)) {
+        qWarning() << "Failed to add source " << resource->flatpakName() << error->message;
         return nullptr;
     }
 
@@ -335,7 +375,9 @@ void FlatpakSourcesBackend::addRemote(FlatpakRemote *remote, FlatpakInstallation
     if (flatpak_installation_get_is_user(installation)) {
         label = i18n("%1 (user)", label);
     }
-    FlatpakSourceItem *it = new FlatpakSourceItem(label);
+
+    FlatpakBackend *backend = qobject_cast<FlatpakBackend *>(parent());
+    FlatpakSourceItem *it = new FlatpakSourceItem(label, remote, backend);
     it->setData(remoteUrl.isLocalFile() ? remoteUrl.toLocalFile() : remoteUrl.host(), Qt::ToolTipRole);
     it->setData(remoteUrl, Qt::StatusTipRole);
     it->setData(id, IdRole);
@@ -343,6 +385,7 @@ void FlatpakSourcesBackend::addRemote(FlatpakRemote *remote, FlatpakInstallation
 #if FLATPAK_CHECK_VERSION(1, 4, 0)
     it->setData(QString::fromUtf8(flatpak_remote_get_icon(remote)), IconUrlRole);
 #endif
+    it->setCheckable(true);
     it->setFlatpakInstallation(installation);
 
     int idx = -1;
