@@ -405,14 +405,61 @@ void PackageKitUpdater::finished(PackageKit::Transaction::Exit exit, uint /*time
     m_transaction = nullptr;
 
     if (!cancel && simulate) {
-        const auto toremove = m_packagesModified.value(PackageKit::Transaction::InfoRemoving);
+        auto toremoveOrig = m_packagesModified.value(PackageKit::Transaction::InfoRemoving);
+        auto toremove = toremoveOrig;
+        auto toinstall = QStringList() << m_packagesModified.value(PackageKit::Transaction::InfoInstalling)
+                                       << m_packagesModified.value(PackageKit::Transaction::InfoUpdating);
+
+        // some backends will treat upgrades as removal + install, which makes for terrible error messages.
+        for (auto it = toremove.begin(), itEnd = toremove.end(); it != itEnd;) {
+            const QString name = PackageKit::Transaction::packageName(*it);
+            auto itInstall = std::find_if(toinstall.begin(), toinstall.end(), [&](const QString &pkgid) {
+                return name == PackageKit::Transaction::packageName(pkgid);
+            });
+            if (itInstall != toinstall.end()) {
+                toinstall.erase(itInstall);
+                it = toremove.erase(it);
+            } else {
+                ++it;
+            }
+        };
+
+        if (PackageKit::Daemon::backendName() == "dnf") {
+            // Fedora has some packages that it uninstalls then eventually creates on its own. No need to
+            // notify about these.
+            toremove = kFilter<typeof(toremove)>(toremove, [](const QString &pkgid) {
+                return !PackageKit::Transaction::packageName(pkgid).startsWith(QLatin1String("kmod"));
+            });
+        }
+
         if (!toremove.isEmpty()) {
-            const auto toinstall = QStringList() << m_packagesModified.value(PackageKit::Transaction::InfoInstalling)
-                                                 << m_packagesModified.value(PackageKit::Transaction::InfoUpdating);
-            Q_EMIT proceedRequest(i18n("Packages to remove"),
-                                  i18n("The following packages will be removed by the update:<ul><li>%1</li></ul><br/>in order to install:<ul><li>%2</li></ul>",
-                                       PackageKitResource::joinPackages(toremove, QStringLiteral("</li><li>"), {}),
-                                       PackageKitResource::joinPackages(toinstall, QStringLiteral("</li><li>"), {})));
+            QStringList criticals;
+            for (const auto &pkgid : toremove) {
+                auto res = kFilter<QVector<AbstractResource *>>(m_backend->resourcesByPackageName(pkgid), [](AbstractResource *res) {
+                    return static_cast<PackageKitResource *>(res)->isCritical();
+                });
+                criticals << kTransform<QStringList>(res, [](AbstractResource *a) {
+                    return a->name();
+                });
+                if (!criticals.isEmpty()) {
+                    break;
+                }
+            }
+
+            if (!criticals.isEmpty()) {
+                const QString msg = i18n(
+                    "This update cannot be completed as it would remove the following software which is critical to the system's operation:<nl/>"
+                    "<ul><li>%1</li></ul><nl/>"
+                    "If you believe this is an error, please report it as a bug to the packagers of your distribution.",
+                    criticals.constFirst());
+                Q_EMIT distroErrorMessage(msg);
+            } else {
+                Q_EMIT proceedRequest(
+                    i18n("Packages to remove"),
+                    i18n("The following packages will be removed by the update:<ul><li>%1</li></ul><br/>in order to install:<ul><li>%2</li></ul>",
+                         PackageKitResource::joinPackages(toremove, QStringLiteral("</li><li>"), {}),
+                         PackageKitResource::joinPackages(toinstall, QStringLiteral("</li><li>"), {})));
+            }
         } else {
             proceed();
         }
@@ -504,10 +551,7 @@ void PackageKitUpdater::errorFound(PackageKit::Transaction::Error err, const QSt
 {
     if (err == PackageKit::Transaction::ErrorNoLicenseAgreement)
         return;
-    QString finalMessage = xi18nc("@info",
-                                  "%1:<nl/><nl/>%2<nl/><nl/>Please report this issue to the packagers of your distribution.",
-                                  PackageKitMessages::errorMessage(err),
-                                  error);
+    QString finalMessage = xi18nc("@info", "%1:<nl/><nl/>%2", PackageKitMessages::errorMessage(err), error);
     Q_EMIT passiveMessage(finalMessage);
     qWarning() << "Error happened" << err << error;
 }
